==> nono-0.4.4.tar.gz: nono-0.4.4/debugger/debugger_memory.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// デバッガのメモリアクセスヘルパー(?)
//

#include "debugger_private.h"

// コンストラクタ
DebuggerMemoryStream::DebuggerMemoryStream(const DebuggerMD *md_)
{
	md = md_;
}

// コンストラクタ (全部入り)
DebuggerMemoryStream::DebuggerMemoryStream(const DebuggerMD *md_,
	const saddr_t& laddr_, MemoryMode access_mode, MMULookupMode lookup_mode)
	: DebuggerMemoryStream(md_)
{
	InitAddr(laddr_, access_mode, lookup_mode);
}

void
DebuggerMemoryStream::InitAddr(const saddr_t& laddr_,
	MemoryMode access_mode, MMULookupMode lookup_mode)
{
	laddr = laddr_;
	// lookup はここで enum から bool にしておく
	lookup = static_cast<bool>(lookup_mode);
	// 物理アドレスの時はこれだけでいいので最初に代入しておく
	paddr = (uint32)laddr;

	if (access_mode == MemoryMode::Logical && md->MMUEnabled()) {
		translate = true;
		// FormatAddr() なしで Fetch/Peek を実行した場合でも、初回に
		// アドレス変換を起こさせるため。SetNBreakpoint() で使う。
		pageover = true;
	}
}

// 現在のアドレスを表示用に整形して str に代入する。
// アドレス変換でバスエラーが起きたら false を返す。
bool
DebuggerMemoryStream::FormatAddr(std::string& str)
{
	// ここは pageover に関係なく必ずアドレス変換を行う。
	// (その結果必ず pageover = false になる)

	if (translate) {
		// laddr は論理アドレスなのでアドレス変換する
		uint64 tmp = md->TranslateAddr(laddr, lookup);
		pageover = false;

		// laddr は常に表示
		str = string_format("%08x", (uint32)laddr);

		if ((int64)tmp < 0) {
			// アドレス変換の結果バスエラーなら false で帰る
			str += ":BusErr";
			return false;
		}

		paddr = (uint32)tmp;
		if (paddr == (uint32)laddr) {
			// PA==VA
			str += "=PA";
		} else {
			// PA!=VA
			str += string_format(":%08x:", paddr);
		}
	} else {
		// laddr は物理アドレスなのでそのまま
		paddr = (uint32)laddr;

		str = string_format("%08x:", paddr);
	}

	return true;
}

// アドレスを進める(戻す)。
void
DebuggerMemoryStream::Offset(int n)
{
	ResetAddr(laddr.GetAddr() + n);
}

// アドレスをリセットする。
// (他のパラメータは維持するので、1ワード戻るとかそういう時用)
void
DebuggerMemoryStream::ResetAddr(uint32 newaddr)
{
	if (((newaddr ^ (uint32)laddr) & 0xffffff00) != 0) {
		pageover = true;
	}
	laddr = newaddr;
	paddr = newaddr;
}

// 前回のアクセスでページ境界をまたいだかも知れない場合は再変換。
#define PRECHECK() do {	\
	if (__predict_false(pageover)) {	\
		uint64 tmp = md->TranslateAddr(laddr, lookup);	\
		if ((int64)tmp < 0) {	\
			return (uint64)-2;	\
		}	\
		paddr = (uint32)tmp;	\
		pageover = false;	\
	}	\
} while (0)

// 論理アドレスがページ境界をまたいだかどうか。
// m68k はページサイズが 256バイト..32KB まで可変だが、いつ変わるか
// 分からないページサイズマスクを取得することを考えたら、多少の無駄
// を承知で常に 256 バイトおきにチェックするほうがマシかもしれない。
#define POSTCHECK() do {	\
	if (__predict_false(translate && (paddr & 0xff) == 0)) {	\
		pageover = true;	\
	}	\
} while (0)


// laddr から単位命令語を読み込むが、アドレスは進めない。
uint64
DebuggerMemoryStream::PeekInst()
{
	uint64 data;

	assert(((uint32)laddr & (md->inst_bytes - 1)) == 0);

	PRECHECK();

	data = 0;
	for (int i = 0; i < md->inst_bytes; i++) {
		data <<= 8;
		data |= vm_phys_peek_8(paddr + i);
	}

	return data;
}

// laddr から単位命令語を読み込んで、アドレスを進める。
uint64
DebuggerMemoryStream::FetchInst()
{
	uint64 data;

	data = PeekInst();
	laddr += md->inst_bytes;
	paddr += md->inst_bytes;

	POSTCHECK();

	return data;
}

// laddr から1バイト読み込んで、アドレスを進める。
uint64
DebuggerMemoryStream::Read8()
{
	uint64 data;

	PRECHECK();

	data = vm_phys_peek_8(paddr);
	laddr++;
	paddr++;

	POSTCHECK();

	return data;
}

==> nono-0.6.3.tar.gz: nono-0.6.3/debugger/debugger_memory.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// デバッガのメモリアクセスヘルパー(?)
//

#include "debugger_memory.h"
#include "debugger_private.h"
#include "mainbus.h"

// コンストラクタ
DebuggerMemoryStream::DebuggerMemoryStream(const DebuggerMD *md_,
	fcaddr laddr_, MemoryMode access_mode_, MMULookupMode lookup_mode)
{
	md = md_;
	assert(md);
	laddr = laddr_;
	paddr = (uint32)laddr;

	bus = md->GetBus();
	lcol = md->GetLASize() / 4;
	pcol = md->GetPASize() / 4;

	translate = false;
	if (access_mode_ == MemoryMode::Logical) {
		// lookup はここで enum から bool にしておく
		lookup = static_cast<bool>(lookup_mode);

		if (md->MMUEnabled()) {
			translate = true;
			// FormatAddr() なしで Fetch/Peek を実行した場合でも、初回に
			// アドレス変換を起こさせるため。SetNBreakpoint() で使う。
			pageover = true;
		}
	}
}

// 現在のアドレスを表示用に整形して str に代入する。
// アドレス変換でバスエラーが起きたら false を返す。
bool
DebuggerMemoryStream::FormatAddr(std::string& str)
{
	// ここは pageover に関係なく必ずアドレス変換を行う。
	// (その結果必ず pageover = false になる)

	if (translate) {
		// laddr は論理アドレスなのでアドレス変換する
		uint64 tmp = md->TranslateAddr(laddr, lookup);
		pageover = false;

		// laddr は常に表示
		str = strhex((uint32)laddr, lcol);

		if ((int64)tmp < 0) {
			// アドレス変換の結果バスエラーなら false で帰る
			str += ":BusErr";
			return false;
		}

		paddr = (uint32)tmp;
		if (paddr == (uint32)laddr) {
			// PA==VA
			str += "=PA";
		} else {
			// PA!=VA
			str += ':';
			str += strhex(paddr, pcol);
			str += ':';
		}
	} else {
		// laddr は物理アドレスなのでそのまま
		paddr = (uint32)laddr;

		str = strhex(paddr, pcol);
		str += ':';
	}

	return true;
}

// アドレスを進める(戻す)。
void
DebuggerMemoryStream::Offset(int n)
{
	ResetAddr(laddr.GetAddr() + n);
}

// アドレスをリセットする。
// (他のパラメータは維持するので、1ワード戻るとかそういう時用)
void
DebuggerMemoryStream::ResetAddr(uint32 newaddr)
{
	if (((newaddr ^ (uint32)laddr) & 0xffffff00) != 0) {
		pageover = true;
	}
	laddr = newaddr;
	paddr = newaddr;
}

// 前回のアクセスでページ境界をまたいだかも知れない場合は再変換。
#define PRECHECK() do {	\
	if (__predict_false(pageover)) {	\
		uint64 tmp = md->TranslateAddr(laddr, lookup);	\
		if ((int64)tmp < 0) {	\
			return (uint64)-2;	\
		}	\
		paddr = (uint32)tmp;	\
		pageover = false;	\
	}	\
} while (0)

// 論理アドレスがページ境界をまたいだかどうか。
// m68k はページサイズが 256バイト..32KB まで可変だが、いつ変わるか
// 分からないページサイズマスクを取得することを考えたら、多少の無駄
// を承知で常に 256 バイトおきにチェックするほうがマシかもしれない。
#define POSTCHECK() do {	\
	if (__predict_false(translate && (paddr & 0xff) == 0)) {	\
		pageover = true;	\
	}	\
} while (0)


// laddr から指定バイト数を右詰めで読み込むが、アドレスは進めない。
uint64
DebuggerMemoryStream::Peek(int bytes)
{
	uint64 data;

	assert(((uint32)laddr & (bytes - 1)) == 0);

	PRECHECK();

	data = 0;
	for (int i = 0; i < bytes; i++) {
		data <<= 8;
		data |= bus->Peek8(paddr + i);
	}

	return data;
}

// laddr から指定バイト数を右詰めで読み込んで、アドレスを進める。
uint64
DebuggerMemoryStream::Read(int bytes)
{
	uint64 data;

	data = Peek(bytes);
	laddr += bytes;
	paddr += bytes;

	POSTCHECK();

	return data;
}

// laddr から1バイト読み込んで、アドレスを進める。
uint64
DebuggerMemoryStream::Read8()
{
	uint64 data;

	PRECHECK();

	data = bus->Peek8(paddr);
	laddr++;
	paddr++;

	POSTCHECK();

	return data;
}
